 ##############################################################################
#
# DicoReverso
# v1.03 (18/04/2020)   ©2012-2020 Menz Agitat
#
# IRC: irc.epiknet.org  #boulets / #eggdrop
#
# Mes scripts sont téléchargeables sur http://www.eggdrop.fr
# Retrouvez aussi toute l'actualité de mes releases sur
# http://wiki.eggdrop.fr/Utilisateur:MenzAgitat
#
 ##############################################################################

#
# DESCRIPTION
#
# Ce script permet d'afficher la définition d'un mot français.
#
# Pour activer DicoReverso sur un salon, vous devez taper ceci en partyline :
# .chanset #NomDuChan +dicoreverso
# et pour le désactiver :
# .chanset #NomDuChan -dicoreverso
#
 ##############################################################################

#
# SYNTAXE
#
# !dico [-collins/-reverso] <mot>
#
#	L'option -collins ou -reverso vous permet de choisir quel dictionnaire
# utiliser. Si vous faites juste !dico <mot>, le réglage default_dictionnary
# (voir section configuration plus bas) déterminera le dictionnaire à utiliser.
# En cas d'absence de définition du Collins, le dictionnaire Reverso sera
# toujours utilisé.
#
 ##############################################################################

#
# CHANGELOG
#
# 1.0
#		- Première version.
# 1.01
#		- Correction : suite à un changement du layout du site, les suggestions
#			orthographiques ne fonctionnaient plus si un mot n'était pas trouvé.
#		- Correction : un "0" s'affichait parfois à la suite de la définition dans
#			certaines conditions.
#		- Correction : la détection de la version d'Eggdrop posait problème sur les
#			versions 1.8+
# 1.02
#		- Correction : le script ne fonctionnait plus suite à un passage du site en
#			https. Le package Tls est désormais requis.
#	1.03
#		- Correction : le script rencontrait une erreur si la version du package
#			http était supérieure ou égale à 2.9.0, suite à une modification du
#			fonctionnement de la procédure ::http::formatQuery.
#		- Correction : les caractères accentués dans le mot à définir pouvaient
#			poser problème.
#		- Correction : le script recontrait une erreur si un mot n'était pas trouvé
#			dans le dictionnaire en ligne et que ce dernier ne proposait aucune
#			suggestion.
#		- Correction : le raccourcissement d'urls avec Tinyurl ne fonctionnait plus.
#		- Correction : suppression d'un espace en trop au début de la définition des
#			mots.
#
 ##############################################################################

#
# LICENCE
#
# Cette création est mise à disposition selon le Contrat
# Attribution-NonCommercial-ShareAlike 3.0 Unported disponible en ligne
# http://creativecommons.org/licenses/by-nc-sa/3.0/ ou par courrier postal à
# Creative Commons, 171 Second Street, Suite 300, San Francisco, California
# 94105, USA.
# Vous pouvez également consulter la version française ici :
# http://creativecommons.org/licenses/by-nc-sa/3.0/deed.fr
#
 ##############################################################################

if {[::tcl::info::commands ::dicoreverso::unload] eq "::dicoreverso::unload"} { ::dicoreverso::unload }
if { [package vcompare [regexp -inline {^[[:digit:]\.]+} $::version] 1.6.20] == -1 } { putloglev o * "\00304\[DicoReverso - erreur\]\003 La version de votre Eggdrop est\00304 ${::version}\003; DicoReverso ne fonctionnera correctement que sur les Eggdrops version 1.6.20 ou supérieure." ; return }
if { [::tcl::info::tclversion] < 8.5 } { putloglev o * "\00304\[DicoReverso - erreur\]\003 DicoReverso nécessite que Tcl 8.5 (ou plus) soit installé pour fonctionner. Votre version actuelle de Tcl est\00304 ${::tcl_version}\003." ; return }
if { [catch { package require tls 1.5 }] } { putloglev o * "\00304\[DicoReverso - erreur\]\003 DicoReverso nécessite le package tls 1.5 (ou plus) pour fonctionner. Le chargement du script a été annulé." ; return }
namespace eval ::dicoreverso {



 ###############################################################################
### Configuration
 ###############################################################################

	# Dictionnaire à utiliser par défaut (1 = Reverso, 2 = Collins)
	# Remarque : si la définition du Collins n'est pas disponible pour un mot, la
	# définition de Reverso sera affichée à la place.
	variable default_dictionnary 1

	# Activer la commande privée ? (1 = oui, 0 = non)
	# Si la commande privée est activée, les utilisateurs pourront demander
	# la définition d'un mot en privé avec le bot en tapant
	# /msg NomDuBot !dico mot
	# Notez qu'il faut se trouver sur au moins un salon en commun avec l'Eggdrop
	# pour pouvoir utiliser cette fonction.
	variable private_mode 0

	# Commande publique
	variable public_cmd "!dico"

	# Autorisations pour la commande publique
	variable public_cmd_auth "-|-"

	# Commande privée
	variable private_cmd "!dico"

	# Autorisations pour la commande privée
	variable private_cmd_auth "-|-"

	# Afficher des suggestions si le mot est mal orthographié ? (0 = non, 1 = oui)
	variable display_suggestions 1

	# Afficher automatiquement la définition de la première suggestion
	# orthographique si le mot est mal orthographié ? (0 = non, 1 = oui)
	variable autodefine_1st_suggestion 0

	# Afficher les mots connexes s'il y en a ? (0 = non, 1 = oui)
	# Exemple : "porte-clés" pour le mot "porte"
	variable display_related 1

	# Nombre maximum de lignes à afficher pour les suggestions
	variable suggestions_max_lines 1

	# Nombre maximum de lignes à afficher pour les définitions
	variable definitions_max_lines 2

	# Nombre maximum de lignes à afficher pour les mots connexes
	variable related_max_lines 1

	# Longueur maximale d'une ligne affichable (si la fin de certaines lignes
	# est tronquée, cela signifie que vous devez diminuer cette valeur)
	variable max_line_length 430

	# Après combien de secondes décide-t-on que le site web utilisé par le script
	# pour afficher les définitions est offline (ou trop lent) en l'absence de
	# réponse de sa part ?
	variable dico_timeout 10

	# Si la définition est trop longue pour être affichée intégralement, doit-on
	# afficher une URL permettant de consulter la version intégrale ?
	# (0 = non, 1 = oui)
	variable show_link_if_truncated 1

	# Raccourcir les URLs affichées avec TinyUrl ? (0 = non, 1 = oui)
	variable shorten_URLs 1

	# Après combien de secondes décide-t-on que le site web www.tinyurl.com est
	# offline (ou trop lent) en l'absence de réponse de sa part ?
	# Remarque : en cas de timeout, l'URL intégrale sera affichée à la place
	# de la TinyURL.
	variable TinyURL_timeout 5

	# URL (n'y touchez pas à moins d'avoir une bonne raison de le faire)
	variable site_URL "https://dictionnaire.reverso.net/francais-definition/\$arg"

	# User-Agent (n'y touchez pas à moins d'avoir une bonne raison de le faire)
	variable useragent "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-GB; rv:1.9.2.2) Gecko/20100316 Firefox/3.6.2"

	# Activer le contrôle de flood ? (0 = non, 1 = oui)
	variable antiflood 1

	# Seuil de déclenchement de l'antiflood.
	# Exemple : "5:90" = 5 commandes maximum en 90 secondes; les suivantes seront
	# ignorées.
	variable flood_threshold "5:90"

	# Intervalle de temps minimum entre l'affichage de 2 messages
	# avertissant que l'antiflood a été déclenché (ne réglez pas
	# cette valeur trop bas afin de ne pas être floodé par les messages
	# d'avertissement de l'antiflood...)
	variable antiflood_msg_interval 20




 ###############################################################################
### Fin de la configuration
 ###############################################################################



	 #############################################################################
	### Initialisation
	 #############################################################################
	package require http
	variable scriptname "DicoReverso"
	variable version "1.03.20200418"
	setudef flag dicoreverso
	variable DEBUGMODE 0
	switch -nocase -- $default_dictionnary {
		1 { set use_dictionnary "ctl00_cC_translate_box" }
		2 { set use_dictionnary "ctl00_cC_collinsResultsDiv" }
		default { set use_dictionnary "ctl00_cC_translate_box" }
	}
	proc unload {args} {
		putlog "Désallocation des ressources de ${::dicoreverso::scriptname}..."
		foreach binding [lsearch -inline -all -regexp [binds *[set ns [::tcl::string::range [namespace current] 2 end]]*] " \{?(::)?$ns"] {
			unbind [lindex $binding 0] [lindex $binding 1] [lindex $binding 2] [lindex $binding 4]
		}
		foreach running_utimer [utimers] {
			if { [::tcl::string::match "*[namespace current]::*" [lindex $running_utimer 1]] } { killutimer [lindex $running_utimer 2] }
		}
		namespace delete ::dicoreverso
	}
}

 ###############################################################################
### Procédure principale
 ###############################################################################
proc ::dicoreverso::pub_process {nick host hand chan arg} {
	::dicoreverso::process $nick $host $hand $chan $arg
}
proc ::dicoreverso::msg_process {nick host hand arg} {
	::dicoreverso::process $nick $host $hand private $arg
}
proc ::dicoreverso::process {nick host hand chan arg}  {
	if { ($::dicoreverso::antiflood) && ([::dicoreverso::antiflood $chan $nick]) } { return }
	if { $chan eq "private" } {
		if { ![onchan $nick] } {
			::dicoreverso::output_message $nick help 0 PRIVMSG "Vous devez vous trouver sur un salon sur lequel je suis aussi pour utiliser le dictionnaire."
			return
		}
		set chan $nick
	} else {
		if { ![channel get $chan dicoreverso] } { return }
	}
	regsub -all -- {\003[0-9]{0,2}(,[0-9]{0,2})?|\017|\037|\002|\026|\006|\007} [split [::tcl::string::trim $arg " "]] "" arg
	if { $arg eq "" } {
		::dicoreverso::output_message $chan help 0 PRIVMSG "\037Syntaxe\037 : \002[set ::dicoreverso::public_cmd]\002 \00314\[\003-collins\00314/\003-reverso\00314\] <\003mot\00314> \00307| \003Cherche la définition d'un mot dans le dictionnaire. L'option -collins ou -reverso vous permet de choisir quel dictionnaire utiliser. En cas d'absence de définition du Collins, le dictionnaire Reverso sera toujours utilisé."
	} else {
		set arg [split $arg]
		if { [set lower_arg [::tcl::string::tolower [lindex $arg 0]]] eq "-collins" } {
			set dictionnary "ctl00_cC_collinsResultsDiv"
			set dictionnary_id 2
			set arg [join [lrange $arg 1 end]]
		} elseif { $lower_arg eq "-reverso" } {
			set dictionnary "ctl00_cC_translate_box"
			set dictionnary_id 1
			set arg [join [lrange $arg 1 end]]
		} else {
			set dictionnary $::dicoreverso::use_dictionnary
			set dictionnary_id $::dicoreverso::default_dictionnary
		}
		::http::register https 443 [list ::tls::socket -tls1 1]
		# on modifie l'urlencoding car dicoreverso.net ne comprend que l'utf-8 dans
		# ses URLs.
		array set httpconfig [::http::config]
		::http::config -urlencoding utf-8 -useragent $::dicoreverso::useragent
		# On remplace les caractères spéciaux par leur équivalent hexadécimal pour
		# pouvoir être utilisés dans l'url.
		set arg [http::mapReply $arg]
		# on restaure l'urlencoding comme il était avant qu'on y touche
		::http::config -urlencoding $httpconfig(-urlencoding)

		if { [catch { set token [::http::geturl [set url [subst $::dicoreverso::site_URL]] -timeout [expr $::dicoreverso::dico_timeout * 1000]] }] } {
			::dicoreverso::output_message $chan help 0 PRIVMSG "\00314La connexion à \00312\037[set url]\037\003\00314 n'a pas pu être établie. Il est possible que le site rencontre un problème technique.\003"
		} elseif {[::http::status $token] eq "ok"} {
			# on extrait la partie qui nous intéresse et sur laquelle on va travailler
			set received_data [::http::data $token]
			regexp {<div class="centerColumn">(.+?)</div>(?:\s+)?<!--Center section end} $received_data received_data
			if {
				([::tcl::info::exists received_data])
				&& ($received_data ne "")
			} then {
				if { $::dicoreverso::DEBUGMODE } {
					set fileID [open "scripts/dicoreverso_raw_output.txt" w]
					puts $fileID "---------------------------- [set url]\n"
					puts $fileID $received_data
					close $fileID
				}
				### suggestions orthographiques
				if { $::dicoreverso::display_suggestions } {
					# extraction de la suggestion orthographique principale
					regexp -nocase {<h2 class='resh2'>(.+?)<} $received_data {} main_suggestion
					# extraction de la partie contenant les autres suggestions orthographiques
					regexp -nocase {<div id="ctl00_cC_translationNotFound"(.+?)(?:<!--  Translation not found  :end  -->)} $received_data suggestions
					if { [::tcl::info::exists suggestions] } {
						# remplacement des sauts de ligne par un espace
						regsub -all {\n} $suggestions " " suggestions
						# filtrage du code html et conversion des caractères spéciaux
						regsub -all {<[^<]*>} [::dicoreverso::filter $suggestions] "" suggestions
						# suppression de "Autres suggestions :" dans les données envoyées par le site web
						regsub {Autres suggestions :} $suggestions "" suggestions
						# suppression des espaces en double
						regsub -all {\s+} $suggestions " " suggestions
						# remplacement des virgules par un autre séparateur
						regsub -all {,\s} $suggestions " \00307-\003 " suggestions
						# suppression des espaces superflus en début ou fin de ligne
						set suggestions [::tcl::string::trim $suggestions " "]
						if {
							($suggestions eq "Votre recherche n'a retourné aucun résultat")
							|| ($suggestions eq "")
							|| ![info exists main_suggestion]
						} then {
							::dicoreverso::output_message $chan help 0 PRIVMSG "\00314Votre recherche n'a retourné aucun résultat.\003"
							::http::cleanup $token
							::http::unregister https
							return
						} else {
							# ajout de la suggestion orthographique principale à la liste des suggestions
							set suggestions "$main_suggestion \00307-\003 $suggestions"
						}
					} else {
						set suggestions 0
					}
				} else {
					set suggestions 0
				}
				### définition
				regexp "<div id=\"[set dictionnary]\"(?:.+?)>(.+?)</font>" $received_data definition
				if { ![::tcl::info::exists definition] } {
					# Si la définition du Collins a été demandée et n'existe pas, on traite
					# la définition de Reverso
					if { $dictionnary_id == 2 } {
						regexp {<div id="ctl00_cC_translate_box"(?:.+?)>(.+?)</font>} $received_data definition
					} else {
						::dicoreverso::output_message $chan help 0 PRIVMSG "\00314Aucune donnée exploitable n'a pu être récupérée. Peut-être le site rencontre-t-il des difficultés techniques ou sa mise en page a changé.\003"
						::http::cleanup $token
						::http::unregister https
						return
					}
				}
				# remplacement des sauts de ligne par un espace
				regsub -all {\n} $definition " " definition
				### mots connexes
				if { [::tcl::string::match "*<hr>*" $definition] } {
					# extraction de la partie contenant les mots connexes
					regexp {<hr>(.+?)</font>} $definition related
					# suppression des la partie contenant les mots connexes dans la définition
					regexp "<div id=\"[set dictionnary]\"(?:.+?)>(.+?)<hr>" $definition definition
					if { $::dicoreverso::display_related } {
						# ajout d'un séparateur entre les différents mots connexes
						regsub -all -nocase {<div border="1" class="ldcomIN"><b> <span style="font-size: (?:[0-9]+?)px;"> <span id="(?:.+?)" style="color:#0000ff;" direction="">(.+?)<span(?:.+?)</div>} $related "\\1 \00307-\003 " related
						# suppression de la définition des mots connexes pour ne garder que les mots connexes eux-mêmes
						regsub -all -nocase {<div border="1" class="ldcomIN"> <span style="font-size: 10px;"> <span id="(?:.+?)" style="color:#ffffff;" direction="">(.+?)</div>} $related "" related
						# filtrage du code html et conversion des caractères spéciaux
						regsub -all {<[^<]*>} [::dicoreverso::filter $related] "" related
						# suppression des espaces en double
						regsub -all {\s+} $related " " related
						# suppression du séparateurs superflu à la fin de la liste de mots connexes
						regsub {\00307-\003\s+?$} $related "" related
						# mise en forme finale
						set related "\037Voir aussi\037 :[set related]"
					} else {
						set related 0
					}
				} else {
					set related 0
				}
				### définition (suite)
				# mise en gras du mot
				regsub -all -nocase {<b>(?:.+?)</b>} $definition "\002\\0\002" definition
				# mise en place des séparateurs entre les différents sens du mot
				regsub -all -nocase {<br></div>} $definition "\\1\00307|\003" definition
				# mise en forme des genres | exemple : (nm)
				regsub -all -nocase {(<span id="(?:.+?)" style="color:#B50000;" direction="">)(.+?)(<span)} $definition "\\1\00314(\\2)\003\\3" definition
				# mise en forme des types | exemple : [familier]
				regsub -all {(<span id="(?:.+?)" style="color:#008000;" direction="">)(.+?)(</span></span>)} $definition "\\1\[\\2\]\\3" definition
				# cas spécial pour [antonyme]
				regsub -all {(<span id="(?:.+?)" style="color:#808080;" direction="">)(.+?)(<)} $definition "\\1 \037\\2\037 :\\3" definition
				regsub -all {\037\[} $definition "\037" definition
				regsub -all {\]\037} $definition "\037" definition
				# suppression des [( )] en double
				regsub -all {\[(?:[^\)\]]+?)\(} $definition "\[" definition
				regsub -all {\)(?:[^/]+?)\]} $definition "\]" definition
				# mise en forme des numéros des différents sens du mot
				regsub -all {<span style="background-color:#000000">([^<]+?)</span>} $definition "\00314\\1 -\003" definition
				# filtrage des scripts
				regsub -all {<script>.+?</script>} $definition "" definition
				# filtrage du code html et conversion des caractères spéciaux
				regsub -all {<[^<]*>} [::dicoreverso::filter $definition] "" definition
				# remplacement du symbole indiquant d'autres orthographes
				regsub -all {\002(?:\s+)?&#8594;} $definition "... ou\002" definition
				# suppression du gras sur les genres | exemple : (nm)
				regsub -all {\002\s+?\00314\((.+?)\)\003\s+?\002} $definition "\00314(\\1)\003" definition
				# peaufinage de l'apparence
				regsub -all {\s+\]} $definition "\]" definition
				regsub -all {\s+\002\s+} $definition "\002 " definition
				regsub -all {\(\s+} $definition "(" definition
				# suppression des espaces en double
				regsub -all {\s+} $definition " " definition
				# peaufinage de l'apparence (suite)
				regsub {\00307\|\003\s+?$} $definition "" definition
				regsub -all {(\s+)?\002(\s+)?\002(\s+)?,} $definition "," definition
				regsub -all {\002\s} $definition "\002" definition
				if { $::dicoreverso::DEBUGMODE } {
					set fileID [open "scripts/dicoreverso_filtered_output.txt" w]
					puts $fileID "---------------------------- [set url]\n"
					puts $fileID "[set suggestions]\n"
					puts $fileID "[set definition]\n"
					puts $fileID $related
					close $fileID
				}
				### affichage du résultat de la recherche
				set has_been_truncated 0
				if { $suggestions ne "0" } {
					if { [::dicoreverso::output_message $chan help $::dicoreverso::suggestions_max_lines PRIVMSG "Aucun résultat exact pour votre recherche. \037Suggestions\037 : [set suggestions]"] } { set has_been_truncated 1 }
				}
				if { ($suggestions eq "0") || (($suggestions ne "0") && ($::dicoreverso::autodefine_1st_suggestion)) } {
					if { [::dicoreverso::output_message $chan help $::dicoreverso::definitions_max_lines PRIVMSG $definition] } { set has_been_truncated 1 }
				}
				if { ($related ne "0") && (($suggestions eq "0") || (($suggestions ne "0") && ($::dicoreverso::autodefine_1st_suggestion))) } {
					if { [::dicoreverso::output_message $chan help $::dicoreverso::related_max_lines PRIVMSG $related] } { set has_been_truncated 1 }
				}
				if { $::dicoreverso::show_link_if_truncated && $has_been_truncated } {
					if { $::dicoreverso::shorten_URLs } {
						set referral_URL [::dicoreverso::tinyurl_conversion $url]
					} else {
						set referral_URL $url
					}	
					::dicoreverso::output_message $chan help 0 PRIVMSG "\00314Vous pouvez consulter la définition intégrale ici : \00312\037$referral_URL\037\003"
				}
			} else {
				::dicoreverso::output_message $chan help 0 PRIVMSG "\00314Aucune donnée exploitable n'a pu être récupérée. Peut-être le site rencontre-t-il des difficultés techniques ou sa mise en page a changé.\003"
			}
			::http::cleanup $token
			::http::unregister https
		}
	}
}

 ###############################################################################
### Affichage d'un message.
### target peut être un nick ou un chan et sera ignoré si output_mode
###		vaut PARTYLINE
### queue peut valoir quick ou help et sera ignoré si output_mode vaut PARTYLINE
### max_lines est un entier numérique et détermine ne nombre maximum de lignes
###   à afficher (0 = aucune limite)
### output_mode peut valoir PRIVMSG NOTICE ou PARTYLINE
### La proc retourne 0 si le texte a été affiché intégralement ou 1 si le texte
###   a été tronqué en raison de la valeur de max_lines
 ###############################################################################
proc ::dicoreverso::output_message {target queue max_lines output_mode data} {
	# Filtrage des styles si nécessaire
	set must_break 0
	if { ([::tcl::string::match "#*" $target]) && ([::tcl::string::match *c* [lindex [split [getchanmode $target]] 0]]) } {
		set monochrome 1
		regsub -all "\017" [stripcodes abcgru $data] "" data
	} else {
		set monochrome 0
	}
	if { $output_mode eq "PARTYLINE" } {
		foreach line [::dicoreverso::split_line $data $::dicoreverso::max_line_length] {
			putloglev o * $line
		}
	} else {
		set counter 0
		set num_lines [llength [set lines [::dicoreverso::split_line $data $::dicoreverso::max_line_length]]]
		foreach line $lines {
			incr counter
			if { ($counter == $max_lines) && ($counter < $num_lines) } {
				if { $monochrome } {
					set line [::tcl::string::replace $line end-5 end "(...)"]
				} else {
					set line [::tcl::string::replace $line end-9 end "\00304(...)\003"]
				}
				set must_break 1
			}
			put[set queue] "[set output_mode] [set target] :[set line]"
			if { $must_break } { break }
		}
	}
	return $must_break
}

 ###############################################################################
### Découpage d'une ligne trop longue en plusieurs lignes en essayant de couper
### sur les espaces autant que possible.
### Les \n provoquent un retour à la ligne.
 ###############################################################################
proc ::dicoreverso::split_line {data {limit 1}} {
	incr limit -1
	set data [::tcl::string::trim $data]
	set data_length [::tcl::string::length $data]
	if { $data_length <= $limit } {
		return [split $data "\n"]
	} else {
		set cursor 0
		# Note : si l'espace le plus proche est situé à plus de 50% de la fin du
		# fragment, on n'hésite pas à couper au milieu d'un mot.
		set middle_pos [expr round($limit / 2.0)]
		while { $cursor < $data_length } {

		if { ([set cut_index [::tcl::string::first "\n" $data $cursor]] != -1)
			&& ($cut_index <= $cursor + $limit)
		} then {
			# on ne fait rien de plus, on vient de définir $cut_index
		} elseif { ([set cut_index [::tcl::string::last " " $data [expr {$cursor + $limit + 1}]]] == -1)
				|| ($cut_index <= $cursor)
				|| ($data_length - $cursor < $limit)
				|| ($cut_index - $cursor < $middle_pos)
			} then {
				set cut_index [expr {$cursor + $limit}]
			}
			lappend output [::tcl::string::trimright [::tcl::string::range $data $cursor $cut_index]]
			set cursor [expr {$cut_index + 1}]
		}
		return $output
	}
}

 ###############################################################################
### Conversion de l'URL en une version plus courte (d'après le script
### tktiny-0.5.tcl de Tomekk)
### Remarque : après un timeout de 10 secondes, l'URL originale est affichée
### par défaut.
 ###############################################################################
proc ::dicoreverso::tinyurl_conversion {url} {
	set url [::tcl::string::map -nocase {"&" "&"} $url]
	set query "http://tinyurl.com/api-create.php?[::http::formatQuery url $url]"
	if { ![catch { set token [::http::geturl $query -timeout [expr {$::dicoreverso::TinyURL_timeout * 1000}]] }] } {
		set short_url [::http::data $token]
		::http::cleanup $token
		if { ($short_url != 0) && ($short_url ne "") } {
			return $short_url
		} else {
			return $url
		}
	} else {
		::http::cleanup $token
		return $url
	}}

 ###############################################################################
### Conversion des entités html
 ###############################################################################
proc ::dicoreverso::filter {data} {
	return [::tcl::string::map {
		"&agrave;"		"à"		"&agrave;"		"à"		"&aacute;"		"á"		"&acirc;"			"â"
		"&atilde;"		"ã"		"&auml;"			"ä"		"&aring;"			"å"		"&aelig;"			"æ"
		"&ccedil;"		"ç"		"&egrave;"		"è"		"&eacute;"		"é"		"&ecirc;"			"ê"
		"&euml;"			"ë"		"&igrave;"		"ì"		"&iacute;"		"í"		"&icirc;"			"î"
		"&iuml;"			"ï"		"&eth;"				"ð"		"&ntilde;"		"ñ"		"&ograve;"		"ò"
		"&oacute;"		"ó"		"&ocirc;"			"ô"		"&otilde;"		"õ"		"&ouml;"			"ö"
		"&divide;"		"÷"		"&oslash;"		"ø"		"&ugrave;"		"ù"		"&uacute;"		"ú"
		"&ucirc;"			"û"		"&uuml;"			"ü"		"&yacute;"		"ý"		"&thorn;"			"þ"
		"&yuml;"			"ÿ"		"&quot;"			"\""	"&amp;"				"&"		"&euro;"			"€"
		"&oelig;"			"œ"		"&Yuml;"			"Ÿ"		"&iexcl;"			"¡"
		"&cent;"			"¢"		"&pound;"			"£"		"&curren;"		"¤"		"&yen;"				"¥"
		"&brvbar;"		"¦"		"&brkbar;"		"¦"		"&sect;"			"§"		"&uml;"				"¨"
		"&die;"				"¨"		"&copy;"			"©"		"&ordf;"			"ª"		"&laquo;"			"«"
		"&not;"				"¬"		"&shy;"				"­-"	"&reg;"				"®"		"&macr;"			"¯"
		"&hibar;"			"¯"		"&deg;"				"°"		"&plusmn;"		"±"		"&sup2;"			"²"
		"&sup3;"			"³"		"&acute;"			"´"		"&micro;"			"µ"		"&para;"			"¶"
		"&middot;"		"·"		"&cedil;"			"¸"		"&sup1;"			"¹"		"&ordm;"			"º"
		"&raquo;"			"»"		"&frac14;"		"¼"		"&frac12;"		"½"		"&frac34;"		"¾"
		"&iquest;"		"¿"		"&Agrave;"		"À"		"&Aacute;"		"Á"		"&Acirc;"			"Â"
		"&Atilde;"		"Ã"		"&Auml;"			"Ä"		"&Aring;"			"Å"		"&AElig;"			"Æ"
		"&Ccedil;"		"Ç"		"&Egrave;"		"È"		"&Eacute;"		"É"		"&Ecirc;"			"Ê"
		"&Euml;"			"Ë"		"&Igrave;"		"Ì"		"&Iacute;"		"Í"		"&Icirc;"			"Î"
		"&Iuml;"			"Ï"		"&ETH;"				"Ð"		"&Dstrok;"		"Ð"		"&Ntilde;"		"Ñ"
		"&Ograve;"		"Ò"		"&Oacute;"		"Ó"		"&Ocirc;"			"Ô"		"&Otilde;"		"Õ"
		"&Ouml;"			"Ö"		"&times;"			"×"		"&Oslash;"		"Ø"		"&Ugrave;"		"Ù"
		"&Uacute;"		"Ú"		"&Ucirc;"			"Û"		"&Uuml;"			"Ü"		"&Yacute;"		"Ý"
		"&THORN;"			"Þ"		"&szlig;"			"ß"		"\r"					"\n"	"\t"					""
		"&#039;"			"\'"	"&#39;"				"\'"	"&nbsp;"			" "		"&nbsp"				" "
		"&#34;"				"\'"	"&#38;"				"&"		"#91;"				"\("	"&#92;"				"\/"
		"&#93;"				")"		"&#123;"			"("		"&#125;"			")"		"&#163;"			"£"
		"&#168;"			"¨"		"&#169;"			"©"		"&#171;"			"«"		"&#173;"			"­"
		"&#174;"			"®"		"&#180;"			"´"		"&#183;"			"·"		"&#185;"			"¹"
		"&#187;"			"»"		"&#188;"			"¼"		"&#189;"			"½"		"&#190;"			"¾"
		"&#192;"			"À"		"&#193;"			"Á"		"&#194;"			"Â"		"&#195;"			"Ã"
		"&#196;"			"Ä"		"&#197;"			"Å"		"&#198;"			"Æ"		"&#199;"			"Ç"
		"&#200;"			"È"		"&#201;"			"É"		"&#202;"			"Ê"		"&#203;"			"Ë"
		"&#204;"			"Ì"		"&#205;"			"Í"		"&#206;"			"Î"		"&#207;"			"Ï"
		"&#208;"			"Ð"		"&#209;"			"Ñ"		"&#210;"			"Ò"		"&#211;"			"Ó"
		"&#212;"			"Ô"		"&#213;"			"Õ"		"&#214;"			"Ö"		"&#215;"			"×"
		"&#216;"			"Ø"		"&#217;"			"Ù"		"&#218;"			"Ú"		"&#219;"			"Û"
		"&#220;"			"Ü"		"&#221;"			"Ý"		"&#222;"			"Þ"		"&#223;"			"ß"
		"&#224;"			"à"		"&#225;"			"á"		"&#226;"			"â"		"&#227;"			"ã"
		"&#228;"			"ä"		"&#229;"			"å"		"&#230;"			"æ"		"&#231;"			"ç"
		"&#232;"			"è"		"&#233;"			"é"		"&#234;"			"ê"		"&#235;"			"ë"
		"&#236;"			"ì"		"&#237;"			"í"		"&#238;"			"î"		"&#239;"			"ï"
		"&#240;"			"ð"		"&#241;"			"ñ"		"&#242;"			"ò"		"&#243;"			"ó"
		"&#244;"			"ô"		"&#245;"			"õ"		"&#246;"			"ö"		"&#247;"			"÷"
		"&#248;"			"ø"		"&#249;"			"ù"		"&#250;"			"ú"		"&#251;"			"û"
		"&#252;"			"ü"		"&#253;"			"ý"		"&#254;"			"þ"		"&#9830;"			""
		"&lt;"				"<"		"&gt;"				">"
	} $data]
}

 ###############################################################################
### Contrôle du flood
### Note :
###   antiflood_msg = 0 : l'antiflood n'a pas encore été déclenché
###   antiflood_msg = 1 : message "antiflood toujours actif" déjà affiché
###   antiflood_msg = 2 : message "antiflood déclenché" déjà affiché
 ###############################################################################
proc ::dicoreverso::antiflood {chan nick} {
	set hash [md5 $chan]
	# l'antiflood est dans un statut neutre, on l'initialise
	if { ![::tcl::info::exists ::dicoreverso::instance($hash)] } {
		set ::dicoreverso::instance($hash) 0
		set ::dicoreverso::antiflood_msg($hash) 0
	}
	lassign [split $::dicoreverso::flood_threshold ":"] max_instances instance_length
	if { $::dicoreverso::instance($hash) >= $max_instances } {
		if { $chan ne "private" } {
			if { !$::dicoreverso::antiflood_msg($hash) } {
				set ::dicoreverso::antiflood_msg($hash) 2
				::dicoreverso::output_message $chan help 0 PRIVMSG "\00304:::\003 \00314Contrôle de flood activé pour la commande \002[set ::dicoreverso::public_cmd]\002 : pas plus de [set max_instances] requête(s) toutes les [set instance_length] secondes.\003"
				if { [set msgresettimer [::dicoreverso::utimerexists "::dicoreverso::antiflood_msg_reset [set hash]"]] ne ""} { killutimer $msgresettimer }
				utimer $::dicoreverso::antiflood_msg_interval [list ::dicoreverso::antiflood_msg_reset $hash]
			} elseif { $::dicoreverso::antiflood_msg($hash) == 1 } {
				set ::dicoreverso::antiflood_msg($hash) 2
				::dicoreverso::output_message $chan help 0 PRIVMSG "\00304:::\003 \00314Le contrôle de flood est toujours actif, merci de patienter.\003"
				if { [set msgresettimer [::dicoreverso::utimerexists "::dicoreverso::antiflood_msg_reset [set hash]"]] ne ""} { killutimer $msgresettimer }
				utimer $::dicoreverso::antiflood_msg_interval [list ::dicoreverso::antiflood_msg_reset $hash]
			}
		} else {
			if { $::dicoreverso::antiflood_msg($hash) != 2 } {
				set ::dicoreverso::antiflood_msg($hash) 2
				::dicoreverso::output_message $nick help 0 PRIVMSG "\00304:::\003 \00314Je reçois actuellement trop de demandes de définitions par message privé (plus de [set max_instances] requête(s) en [set instance_length] secondes). Veuillez réessayer d'ici quelques instants.\003"
				if { [set msgresettimer [::dicoreverso::utimerexists "::dicoreverso::antiflood_msg_reset [set hash]"]] ne ""} { killutimer $msgresettimer }
				utimer $::dicoreverso::antiflood_msg_interval [list ::dicoreverso::antiflood_msg_reset $hash]
			}
		}
		return "1"
	} else {
		incr ::dicoreverso::instance($hash)
		utimer $instance_length [list ::dicoreverso::antiflood_close_instance $hash]
		return "0"
	}
}
proc ::dicoreverso::antiflood_close_instance {hash} {
	incr ::dicoreverso::instance($hash) -1
	# si le nombre d'instances retombe à 0, on efface les variables instance et
	# antiflood_msg afin de ne pas encombrer la mémoire inutilement
	if { $::dicoreverso::instance($hash) == 0 } {
		unset ::dicoreverso::instance($hash)
		unset ::dicoreverso::antiflood_msg($hash)
	# le nombre d'instances est retombé en dessous du seuil critique, on
	# réinitialise antiflood_msg
	} else {
		set ::dicoreverso::antiflood_msg($hash) 0
		if { [set msgresettimer [::dicoreverso::utimerexists "::dicoreverso::antiflood_msg_reset [set hash]"]] ne ""} { killutimer $msgresettimer }
	}
}
proc ::dicoreverso::antiflood_msg_reset {hash} {
	set ::dicoreverso::antiflood_msg($hash) 1
}

 ###############################################################################
### Test de l'existence d'un utimer, renvoi de son ID
### (issu de toolbox.tcl de cmwagner <cmwagner@sodre.net>)
 ###############################################################################
proc ::dicoreverso::utimerexists {command} {
	foreach i [utimers] {
		if { ![::tcl::string::compare $command [lindex $i 1]] } then {
			return [lindex $i 2]
		}
	}
}

 ###############################################################################
### Binds
 ###############################################################################
bind pub $::dicoreverso::public_cmd_auth $::dicoreverso::public_cmd ::dicoreverso::pub_process
if {$::dicoreverso::private_mode} {
	bind msg $::dicoreverso::private_cmd_auth $::dicoreverso::private_cmd ::dicoreverso::msg_process
}
bind evnt - prerehash ::dicoreverso::unload 


putlog "$::dicoreverso::scriptname v$::dicoreverso::version (©2012-2020 Menz Agitat) a été chargé."
